@@ -83,13 +83,14 @@ class Line(metaclass=PoolMeta):
 
     def get_supply_state(self, name):
         if self.purchase_request is not None:
-            purchase_line = self.purchase_request.purchase_line
-            if purchase_line is not None:
-                purchase = purchase_line.purchase
-                if purchase.state == 'cancelled':
-                    return 'cancelled'
-                elif purchase.state in ('processing', 'done'):
-                    return 'supplied'
+            if self.purchase_request.state == 'cancelled':
+                return 'cancelled'
+            else:
+                purchase_line = self.purchase_request.purchase_line
+                if purchase_line is not None:
+                    purchase = purchase_line.purchase
+                    if purchase.state in {'processing', 'done'}:
+                        return 'supplied'
             return 'requested'
         return ''
 
@@ -182,6 +183,7 @@ class Line(metaclass=PoolMeta):
         Uom = pool.get('product.uom')
         Move = pool.get('stock.move')
         ShipmentOut = pool.get('stock.shipment.out')
+        Location = pool.get('stock.location')
 
         if self.supply_state != 'supplied':
             return
@@ -221,11 +223,33 @@ class Line(metaclass=PoolMeta):
                 for inv_move in inventory_moves:
                     if inv_move.product == self.product:
                         moves.add(inv_move)
+
+        child_locations = {}
         to_write = []
         to_assign = []
         for move in moves:
             if move.state != 'draft':
                 continue
+
+            childs = child_locations.get(move.from_location)
+            if childs is None:
+                childs = Location.search([
+                        ('parent', 'child_of', [move.from_location.id]),
+                        ('type', '!=', 'view'),
+                        ])
+                child_locations[move.from_location] = childs
+            # Prevent picking from the destination location
+            try:
+                childs.remove(move.to_location)
+            except ValueError:
+                pass
+            # Try first to pick from source location
+            try:
+                childs.remove(move.from_location)
+                childs.insert(0, move.from_location)
+            except ValueError:
+                # from_location may be a view
+                pass
             qties_converted = []
             for key, quantity in quantities.items():
                 move_key = get_key(move, key[0])
